import { EventEmitter } from 'eventemitter3';

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  Object.defineProperty(Constructor, "prototype", {
    writable: false
  });
  return Constructor;
}

var CanvasElement = /*#__PURE__*/function () {
  function CanvasElement(ctx) {
    _classCallCheck(this, CanvasElement);

    this.width = void 0;
    this.height = void 0;
    this.isCanvasElement = true;
    this.context = void 0;
    this.emitter = new EventEmitter();
    this.context = ctx; // 有可能是 node canvas 创建的 context 对象

    var canvas = ctx.canvas || {};
    this.width = canvas.width || 0;
    this.height = canvas.height || 0;
  }

  _createClass(CanvasElement, [{
    key: "getContext",
    value: function getContext(contextId, contextAttributes) {
      return this.context;
    }
  }, {
    key: "getBoundingClientRect",
    value: function getBoundingClientRect() {
      var width = this.width;
      var height = this.height; // 默认都处理成可视窗口的顶部位置

      return {
        top: 0,
        right: width,
        bottom: height,
        left: 0,
        width: width,
        height: height,
        x: 0,
        y: 0
      };
    }
  }, {
    key: "addEventListener",
    value: function addEventListener(type, listener, options) {
      // TODO: implement options
      this.emitter.on(type, listener);
    }
  }, {
    key: "removeEventListener",
    value: function removeEventListener(type, listener, options) {
      this.emitter.off(type, listener);
    }
    /**
     * @see https://www.w3.org/TR/DOM-Level-2-Events/events.html#Events-EventTarget-dispatchEvent
     */

  }, {
    key: "dispatchEvent",
    value: function dispatchEvent(e) {
      this.emitter.emit(e.type, e);
      return true;
    }
  }]);

  return CanvasElement;
}();

function supportEventListener(canvas) {
  if (!canvas) {
    return false;
  } // 非 HTMLCanvasElement


  if (canvas.nodeType !== 1 || !canvas.nodeName || canvas.nodeName.toLowerCase() !== 'canvas') {
    return false;
  } // 微信小程序canvas.getContext('2d')时也是CanvasRenderingContext2D
  // 也会有ctx.canvas, 而且nodeType也是1，所以还要在看下是否支持addEventListener


  var support = false;

  try {
    canvas.addEventListener('eventTest', function () {
      support = true;
    });
    canvas.dispatchEvent(new Event('eventTest'));
  } catch (error) {
    support = false;
  }

  return support;
}

function createMobileCanvasElement(ctx) {
  if (!ctx) {
    return null;
  }

  if (supportEventListener(ctx.canvas)) {
    return ctx.canvas;
  }

  return new CanvasElement(ctx);
}

export { createMobileCanvasElement };
